/*
	Copyright (c) 1995, 1996 Jeff Chua -- All Rights Reserved
*/

#include <stdio.h>		/* printf */
#include <errno.h>		/* errno */
#include <sys/types.h>		/* open, read */
#include <sys/stat.h>		/* open */
#include <fcntl.h>		/* open */
#include <unistd.h>		/* read */
#include <string.h>		/* strstr */
#include <forms.h>


#define Perror(s)		{ \
					if(errno == EINTR) \
						return 8; \
					printf("Line %d:  " \
						"%s:  Error %d:  %s\n", \
						__LINE__, s, errno, \
						sys_errlist[errno]); \
					exit(1); \
				}



#define INTERVAL	1	/* update interval */


			/* fields from /usr/src/linux/char/apm_bios.c */
			/* modified ... */
#define FORMAT		"%f %f %x %x %x %x %s %d %s\n"

#define POINTERS	&driver_version, \
			&apm_bios_info_version, \
			&apm_bios_info_flags, \
			&ac_line_status, \
			&battery_status, \
			&battery_flag, \
			percentage, \
			&time_units, \
			units


double	driver_version,
	apm_bios_info_version;

int	apm_bios_info_flags,
	ac_line_status,
	battery_status,
	battery_flag,
	time_units;

char	percentage[10],
	units[10];




FL_FORM *apm_form;
FL_OBJECT *apm_life, *apm_status;

void create_form(void);


int main(int argc, char *argv[])
{
	int fd, n;
	char buf[256], *p, *q;
	char status[20], life[10];
	char s[4];
	int count = 1;

	if((fd = open("/proc/apm", O_RDONLY)) == EOF)
		Perror("open");


	fl_initialize(argv[0],"xapm",0,0,&argc,argv);

	create_form();


	while(1) {
		switch(n = read(fd, buf, sizeof buf)) {
			case EINTR:
				printf("Interrupted!\n");
				exit(1);
			case 0:
				/* end of file */
				break;
			case EOF:
				Perror("read");
			default:
				switch(sscanf(buf, FORMAT, POINTERS)) {
					case 0:
						printf("invalid input "
							"character\n");
						exit(2);
					case EOF:
						Perror("sscanf");
					default:
						break;
				}

						

				fl_set_slider_value(apm_life, atoi(percentage));
				fl_set_object_label(apm_status, p);

				if(count++ % 2) {
					p = ac_line_status ?
						"Charging" : "Offline";
					fl_set_object_label(apm_status, p);
				}
				else {
					p = percentage;
					fl_set_object_label(apm_status, p);
				}

				fl_check_forms();

				break;
		}

		sleep(INTERVAL);

		if(lseek(fd, 0, SEEK_SET) == EOF)
			Perror("lseek");
	}

}


void create_form(void)
{
  apm_form = fl_bgn_form(FL_UP_BOX, 63, 43);

  apm_life = fl_add_slider(FL_HOR_FILL_SLIDER,6,6,50,15,"");
	fl_set_object_color(apm_life, FL_FREE_COL2, FL_FREE_COL3);
	fl_set_object_lcol(apm_life,FL_BLACK);
	fl_set_object_lalign(apm_life,FL_ALIGN_RIGHT);
	fl_set_slider_bounds(apm_life, 0, 100);

  apm_status = fl_add_box(FL_FRAME_BOX,6,23,51,14,"Status");
	fl_set_object_color(apm_status, FL_FREE_COL1, FL_BLUE);
	fl_set_object_lsize(apm_status,FL_TINY_SIZE);
	fl_set_object_lstyle(apm_status,FL_NORMAL_STYLE);
	fl_set_object_lcol(apm_status, FL_PALEGREEN);	/* apm status text */

  fl_mapcolor(FL_FREE_COL1, 120, 100, 40);
  fl_mapcolor(FL_FREE_COL2, 90, 90, 90);
  fl_mapcolor(FL_FREE_COL3, 50, 50, 150);

  fl_mapcolor(FL_COL1, 100, 100, 100);		/* form */
  fl_mapcolor(FL_MCOL, 0, 0, 240);		/* ??? */
  fl_mapcolor(FL_TOP_BCOL, 140, 140, 140);	/* top border */
  fl_mapcolor(FL_BOTTOM_BCOL, 50, 50, 50);	/* bottom border */
  fl_mapcolor(FL_LEFT_BCOL, 120, 120, 120);	/* left border */


  fl_end_form();
  fl_set_form_position(apm_form, 2, -44);
  fl_show_form(apm_form, FL_PLACE_GEOMETRY, FL_TRANSIENT, "xapm");
}
